library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity timer is
  port (
    reset : in std_logic;
    clk   : in std_logic; -- 50 MHz

    read       : in  std_logic;
    write      : in  std_logic;
    chipselect : in  std_logic;
    address    : in  unsigned(4 downto 0):=(others=>'0');
    readdata   : out unsigned(15 downto 0):=(others=>'0');
    writedata  : in  unsigned(15 downto 0):=(others=>'0');
    irq: out std_logic ;

    key: in std_logic_vector(3 downto 0):=(others=>'0');

    LEDR : out std_logic_vector(17 downto 0);
    LEDG : out std_logic_vector(8 downto 0)
  );
end timer;

architecture rtl of timer is

COMPONENT counter1
  port(
    Clk, Reset: in std_logic;
    Q: out integer
  );
end COMPONENT;

  signal secondCounter: unsigned(25 downto 0) := (others => '0');
  signal onSwitch: std_logic :='0';
  signal resetCounter: std_logic :='0';
  signal countervalue: integer;

  constant IRQ_DOWN : unsigned := "00000";

begin

  comp: counter1 PORT MAP(
    clk => clk,
    reset => resetCounter,
    q => countervalue);

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '0' then
        if chipselect = '1' then
          if write = '1' then
            if address = IRQ_DOWN then
              irq <= '0';
            end if;
          end if;
        end if;
      end if;

      -- TODO: are there too many end if;s here?
      -- everything below is not under if reset = '0'

      secondCounter <= secondCounter + 1;
      if secondCounter = x"2FAF080" then
        irq <= '1';
        readdata <= (others => '0');
        secondCounter <= (others => '0');
        if onSwitch = '1' then
          onSwitch <= '0';
        else
          onSwitch<='1';
        end if;
      end if;

      if Key = "1110" and countervalue = 50000000 then
        irq <= '1';
        resetCounter<='1';
        readdata(15 downto 4) <= (others => '0');
        readdata(3 downto 0) <= unsigned(Key);
      elsif Key = "1101" and countervalue = 50000000 then
        irq <= '1';
        resetCounter <= '1';
        readdata(15 downto 4) <= (others => '0');
        readdata(3 downto 0) <= unsigned(Key);
      elsif Key = "1011" and countervalue = 50000000 then
        irq <= '1';
        resetCounter<='1';
        readdata(15 downto 4)<=(others=>'0');
        readdata(3 downto 0)<=unsigned(Key);
      elsif Key = "0111" and countervalue = 50000000 then
        irq <= '1';
        resetCounter <= '1';
        readdata(15 downto 4) <= (others => '0');
        readdata(3 downto 0) <= unsigned(Key);
      else
        resetCounter <= '0';
      end if;
    end if;
  end process;
end rtl;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity counter1 is
port(
Clk, Reset : in std_logic;
Q : out integer
);
end counter1;

architecture imp of counter1 is
  signal count : integer range 0 to 50000001 := 0;
begin
  process (Clk,Reset,count)
  begin
    if Reset = '1' then
      Q <= 0;
    else
      Q <= count;
    end if;

    if rising_edge(Clk) then
      if Reset = '1' then
        count <= 0;
      else
        if count < 50000000 then
          count <= count + 1;
        end if;
      end if;
    end if;
  end process;
end imp;
